# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/friendlly',
                'doc_host': 'https://xl0.github.io',
                'git_url': 'https://github.com/xl0/friendlly',
                'lib_path': 'friendlly'},
  'syms': { 'friendlly.capture_display': { 'friendlly.capture_display.PassthroughCapturer': ( 'capture_display.html#passthroughcapturer',
                                                                                              'friendlly/capture_display.py'),
                                           'friendlly.capture_display.PassthroughCapturer.__init__': ( 'capture_display.html#passthroughcapturer.__init__',
                                                                                                       'friendlly/capture_display.py'),
                                           'friendlly.capture_display.PassthroughCapturer.clear_output': ( 'capture_display.html#passthroughcapturer.clear_output',
                                                                                                           'friendlly/capture_display.py'),
                                           'friendlly.capture_display.PassthroughCapturer.get_outputs': ( 'capture_display.html#passthroughcapturer.get_outputs',
                                                                                                          'friendlly/capture_display.py'),
                                           'friendlly.capture_display.PassthroughCapturer.publish': ( 'capture_display.html#passthroughcapturer.publish',
                                                                                                      'friendlly/capture_display.py'),
                                           'friendlly.capture_display.PassthroughCapturer.start': ( 'capture_display.html#passthroughcapturer.start',
                                                                                                    'friendlly/capture_display.py'),
                                           'friendlly.capture_display.PassthroughCapturer.stop': ( 'capture_display.html#passthroughcapturer.stop',
                                                                                                   'friendlly/capture_display.py')},
            'friendlly.capture_io': { 'friendlly.capture_io.TeeIO': ('capture_io.html#teeio', 'friendlly/capture_io.py'),
                                      'friendlly.capture_io.TeeIO.__getattr__': ( 'capture_io.html#teeio.__getattr__',
                                                                                  'friendlly/capture_io.py'),
                                      'friendlly.capture_io.TeeIO.__init__': ('capture_io.html#teeio.__init__', 'friendlly/capture_io.py'),
                                      'friendlly.capture_io.TeeIO.__setattr__': ( 'capture_io.html#teeio.__setattr__',
                                                                                  'friendlly/capture_io.py'),
                                      'friendlly.capture_io.TeeIO.getvalue': ('capture_io.html#teeio.getvalue', 'friendlly/capture_io.py'),
                                      'friendlly.capture_io.TeeIO.write': ('capture_io.html#teeio.write', 'friendlly/capture_io.py')},
            'friendlly.capture_trace': { 'friendlly.capture_trace.SaveTraceback': ( 'capture_trace.html#savetraceback',
                                                                                    'friendlly/capture_trace.py'),
                                         'friendlly.capture_trace.SaveTraceback.__init__': ( 'capture_trace.html#savetraceback.__init__',
                                                                                             'friendlly/capture_trace.py'),
                                         'friendlly.capture_trace.SaveTraceback._showtraceback': ( 'capture_trace.html#savetraceback._showtraceback',
                                                                                                   'friendlly/capture_trace.py'),
                                         'friendlly.capture_trace.SaveTraceback.register': ( 'capture_trace.html#savetraceback.register',
                                                                                             'friendlly/capture_trace.py'),
                                         'friendlly.capture_trace.SaveTraceback.unregister': ( 'capture_trace.html#savetraceback.unregister',
                                                                                               'friendlly/capture_trace.py')},
            'friendlly.config': { 'friendlly.config.handle_config': ('config.html#handle_config', 'friendlly/config.py'),
                                  'friendlly.config.strip_comments': ('config.html#strip_comments', 'friendlly/config.py')},
            'friendlly.display': { 'friendlly.display.PassthroughCapturer': ('display.html#passthroughcapturer', 'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.__init__': ( 'display.html#passthroughcapturer.__init__',
                                                                                       'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.clear_output': ( 'display.html#passthroughcapturer.clear_output',
                                                                                           'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.get_outputs': ( 'display.html#passthroughcapturer.get_outputs',
                                                                                          'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.publish': ( 'display.html#passthroughcapturer.publish',
                                                                                      'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.start': ( 'display.html#passthroughcapturer.start',
                                                                                    'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.stop': ( 'display.html#passthroughcapturer.stop',
                                                                                   'friendlly/display.py')},
            'friendlly.magic_cell': { 'friendlly.magic_cell.CellChat': ('magic_cell.html#cellchat', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.__init__': ( 'magic_cell.html#cellchat.__init__',
                                                                                  'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.delete_markdown': ( 'magic_cell.html#cellchat.delete_markdown',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.go': ('magic_cell.html#cellchat.go', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.update_code': ( 'magic_cell.html#cellchat.update_code',
                                                                                     'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.update_code_final': ( 'magic_cell.html#cellchat.update_code_final',
                                                                                           'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.update_markdown': ( 'magic_cell.html#cellchat.update_markdown',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NBClassicChat': ('magic_cell.html#nbclassicchat', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NBClassicChat.__init__': ( 'magic_cell.html#nbclassicchat.__init__',
                                                                                       'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NBClassicChat.delete_markdown': ( 'magic_cell.html#nbclassicchat.delete_markdown',
                                                                                              'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NBClassicChat.update_code': ( 'magic_cell.html#nbclassicchat.update_code',
                                                                                          'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NBClassicChat.update_code_final': ( 'magic_cell.html#nbclassicchat.update_code_final',
                                                                                                'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NBClassicChat.update_markdown': ( 'magic_cell.html#nbclassicchat.update_markdown',
                                                                                              'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NoJSChat': ('magic_cell.html#nojschat', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NoJSChat.__init__': ( 'magic_cell.html#nojschat.__init__',
                                                                                  'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NoJSChat.delete_markdown': ( 'magic_cell.html#nojschat.delete_markdown',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NoJSChat.update_code': ( 'magic_cell.html#nojschat.update_code',
                                                                                     'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NoJSChat.update_code_final': ( 'magic_cell.html#nojschat.update_code_final',
                                                                                           'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.NoJSChat.update_markdown': ( 'magic_cell.html#nojschat.update_markdown',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.cells_to_messages': ( 'magic_cell.html#cells_to_messages',
                                                                                  'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.fr_cell': ('magic_cell.html#fr_cell', 'friendlly/magic_cell.py')},
            'friendlly.magic_line': { 'friendlly.magic_line.fr_line': ('magic_line.html#fr_line', 'friendlly/magic_line.py'),
                                      'friendlly.magic_line.parse_cell': ('magic_line.html#parse_cell', 'friendlly/magic_line.py')},
            'friendlly.notebook': { 'friendlly.notebook.detect_environment': ('notebook.html#detect_environment', 'friendlly/notebook.py'),
                                    'friendlly.notebook.inject_js': ('notebook.html#inject_js', 'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_add_cell': ('notebook.html#nbclassic_add_cell', 'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_delete_cell': ( 'notebook.html#nbclassic_delete_cell',
                                                                                  'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_execute_cell': ( 'notebook.html#nbclassic_execute_cell',
                                                                                   'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_get_cells': ( 'notebook.html#nbclassic_get_cells',
                                                                                'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_patch_kernel': ( 'notebook.html#nbclassic_patch_kernel',
                                                                                   'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_render_cell': ( 'notebook.html#nbclassic_render_cell',
                                                                                  'friendlly/notebook.py'),
                                    'friendlly.notebook.nbclassic_update_cell': ( 'notebook.html#nbclassic_update_cell',
                                                                                  'friendlly/notebook.py'),
                                    'friendlly.notebook.update_code_next': ('notebook.html#update_code_next', 'friendlly/notebook.py'),
                                    'friendlly.notebook.update_code_self': ('notebook.html#update_code_self', 'friendlly/notebook.py'),
                                    'friendlly.notebook.vscode_extract_path': ( 'notebook.html#vscode_extract_path',
                                                                                'friendlly/notebook.py'),
                                    'friendlly.notebook.vscode_get_cells': ('notebook.html#vscode_get_cells', 'friendlly/notebook.py')},
            'friendlly.utils': { 'friendlly.utils.GenericEncoder': ('utils.html#genericencoder', 'friendlly/utils.py'),
                                 'friendlly.utils.GenericEncoder.default': ('utils.html#genericencoder.default', 'friendlly/utils.py'),
                                 'friendlly.utils.cell_to_blocks': ('utils.html#cell_to_blocks', 'friendlly/utils.py'),
                                 'friendlly.utils.display_data_to_blocks': ('utils.html#display_data_to_blocks', 'friendlly/utils.py'),
                                 'friendlly.utils.make_cell_from_captured': ('utils.html#make_cell_from_captured', 'friendlly/utils.py'),
                                 'friendlly.utils.merge_messages': ('utils.html#merge_messages', 'friendlly/utils.py'),
                                 'friendlly.utils.nict': ('utils.html#nict', 'friendlly/utils.py'),
                                 'friendlly.utils.nict.__delattr__': ('utils.html#nict.__delattr__', 'friendlly/utils.py'),
                                 'friendlly.utils.nict.__dir__': ('utils.html#nict.__dir__', 'friendlly/utils.py'),
                                 'friendlly.utils.nict.__getattr__': ('utils.html#nict.__getattr__', 'friendlly/utils.py'),
                                 'friendlly.utils.nict.__init__': ('utils.html#nict.__init__', 'friendlly/utils.py'),
                                 'friendlly.utils.nict.__setattr__': ('utils.html#nict.__setattr__', 'friendlly/utils.py'),
                                 'friendlly.utils.nict._to_dict': ('utils.html#nict._to_dict', 'friendlly/utils.py'),
                                 'friendlly.utils.parse_cell_line': ('utils.html#parse_cell_line', 'friendlly/utils.py'),
                                 'friendlly.utils.safe_int': ('utils.html#safe_int', 'friendlly/utils.py'),
                                 'friendlly.utils.strip_traceback_junk': ('utils.html#strip_traceback_junk', 'friendlly/utils.py'),
                                 'friendlly.utils.svg_to_png': ('utils.html#svg_to_png', 'friendlly/utils.py')}}}
